package com.nisum.example.controller;

import com.nisum.example.constants.ProductConstants;
import com.nisum.example.mapper.ProductMapper;
import com.nisum.example.model.Product;
import com.nisum.example.service.IProductService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import javax.validation.Valid;

@RestController
@RequestMapping(value = ProductConstants.PRODUCT_SERVICE_ROOT,produces = MediaType.APPLICATION_JSON_VALUE)
@Slf4j
@Validated
public class ProductController {

    @Autowired
    private ProductMapper productMapper;

    @Autowired
    private IProductService productService;

    @PostMapping(value = "/product-create",consumes = MediaType.APPLICATION_JSON_VALUE)
    @ResponseStatus(HttpStatus.CREATED)
    public Mono<Product> createProduct(@Valid @RequestBody Product product){
        log.info("Calling service saveProduct() ...");
       return productService.saveProduct(productMapper.toDto(product))
               .map(productMapper::toModel)
               .log();
    }

    @GetMapping(value = "/product-get/{id}",produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseStatus(HttpStatus.OK)
    public Mono<Product> fetchProduct(@PathVariable  String id){
        log.info("Calling service getProduct()...");
        return productService.getProduct(id)
                .map(productMapper::toModel)
                .log();
    }

    @GetMapping(value = "/product-all",produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseStatus(HttpStatus.FOUND)
    public Flux<Product> fetchProducts(){
        log.info("Calling service getProducts()...");
        return productService.getProducts()
                .map(productMapper::toModel)
                .log();
    }

    @PutMapping(value = "/product-update",consumes = MediaType.APPLICATION_JSON_VALUE)
    @ResponseStatus(HttpStatus.OK)
    public Mono<Product> modifyProduct(@Valid @RequestBody Product product){
        log.info("Calling service updateProduct()...");
        return productService.updateProduct(productMapper.toDto(product))
                .map(productMapper::toModel)
                .log();
    }

    @DeleteMapping(value = "/product-delete/{id}",produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseStatus(HttpStatus.OK)
    public Mono<Boolean> deleteProductById(@PathVariable String id){
        log.info("Calling service removeById()...");
        return productService.removeById(id)
                .log();
    }

    @PutMapping("/product-update-price")
    @ResponseStatus(HttpStatus.OK)
    public Mono<Product> modifyProductPrice(@RequestParam("prodId") String id,
                                            @RequestParam("prodPrice") Double prodPrice){
        log.info("prodId:"+id+" "+"prodPrice "+prodPrice);
        log.info("Calling service updateProductPrice()...");
        return productService.updateProductPrice(id,prodPrice)
                .map(productMapper::toModel)
                .log();
    }

    @GetMapping("/product-get-name/{name}")
    @ResponseStatus(HttpStatus.OK)
    public Flux<Product> fetchProductByName(@PathVariable("name") String prodName){
        log.info("Calling service getProductByName()...");
        return productService.getProductByName(prodName)
                .map(productMapper::toModel)
                .log();

    }
}
