package com.nisum.example.service;

import com.nisum.example.dto.ProductDTO;
import com.nisum.example.repository.ProductRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.ReactiveMongoTemplate;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

/**
 * Product Service class
 */
@Service
@Slf4j
public class ProductServiceImpl implements IProductService {

    @Autowired
    private ReactiveMongoTemplate reactiveMongoTemplate;

    @Autowired
    private ProductRepository productRepository;

    /**
     *Perform save operation of one product
     * @param productDTO
     * @return Mono<ProductDTO>
     */
    @Override
    public Mono<ProductDTO> saveProduct(ProductDTO productDTO) {
        log.info("Calling repository save()...");
        return productRepository.save(productDTO);
    }

    /**
     * Perform get operation based on id
     * @param id
     * @return Mono<ProductDTO>
     */
    @Override
    public Mono<ProductDTO> getProduct(String id) {
        log.info("Calling repository findById()...");
        return productRepository.findById(id);
    }

    @Override
    public Flux<ProductDTO> getProducts() {
        log.info("Calling repository findAll()...");
        return productRepository.findAll();
    }

    @Override
    public Mono<ProductDTO> updateProduct(ProductDTO productDTO) {
        log.info("Calling repository save()...");
        return productRepository.save(productDTO);
    }

    @Override
    public Mono<Boolean> removeById(String id) {
        log.info("Calling repository clearById()...");
        return productRepository.clearProductById(id);
    }

    @Override
    public Mono<ProductDTO> updateProductPrice(String id,Double prodPrice) {
        log.info("Calling repository changePrice()...");
        return productRepository.changeProductPrice(id,prodPrice);
    }

    @Override
    public Flux<ProductDTO> getProductByName(String prodName) {
        log.info("Calling repository pickProductByName()...");
        return productRepository.pickProductByName(prodName);
    }
}
